using System;
using System.Text.RegularExpressions;

namespace DCLJobView
{
	/// <summary>
	/// Error patterns for a language. Includes error, warning and extra line regexps.
	/// Idea and basic algorithm stolen from jEdit Console plugin.
	/// 
	/// The following names apply to partial matches (like "(?<name>)") of the
	/// Error and Warning regexps:
	///		filename      The name of the file with the error (optional).
	///		line          The line number of the error.
	///		column        The line column of the error (optional).
	///		message       The description of the errror.
	///		
	///	The ExtraLine regexp has the following name:
	///		extra         The extra message.
	/// </summary>
	[Serializable]
	public class CompilerErrorPattern
	{
		[NonSerialized]
		private Regex error = null;
		[NonSerialized]
		private Regex warning = null;
		[NonSerialized]
		private Regex extraLine = null;

		private string name;
		private string errorRegexp;
		private string warningRegexp;
		private string extraLineRegexp;

		public string Name
		{
			get { return this.name; }
			set { this.name = value; }
		}

		public string ErrorRegexp
		{
			get { return this.errorRegexp; }
			set { this.errorRegexp = value; }
		}

		public string WarningRegexp
		{
			get { return this.warningRegexp; }
			set { this.warningRegexp = value; }
		}

		public string ExtraLineRegexp
		{
			get { return this.extraLineRegexp; }
			set { this.extraLineRegexp = value; }
		}

		public CompilerErrorPattern(CompilerErrorPattern cep)
		{
			this.name = cep.name;
			this.errorRegexp = cep.errorRegexp;
			this.warningRegexp = cep.warningRegexp;
			this.extraLineRegexp = cep.extraLineRegexp;

			// we do not compile here, since this copy constructor is usually called in config environnent
		}

		public CompilerErrorPattern(string name, string errorRegexp, string warningRegexp, string extraLineRegexp)
		{
			this.name = name;
			this.errorRegexp = errorRegexp;
			this.warningRegexp = warningRegexp;
			this.extraLineRegexp = extraLineRegexp;

			this.CompileRegexps();
		}

		private void CompileRegexps()
		{
			if (this.error == null && this.errorRegexp != null && this.errorRegexp != "")
				this.error = new Regex(this.errorRegexp);

			if (this.warning == null && this.warningRegexp != null && this.warningRegexp != "")
				this.warning = new Regex(this.warningRegexp);

			if (this.extraLine == null && this.extraLineRegexp != null && this.extraLineRegexp != "")
				this.extraLine = new Regex(this.extraLineRegexp);
		}

		public CompilerOutputItem Match(string line)
		{
			this.CompileRegexps();

			Match match = null;
			bool success = false;
			
			if (this.warning != null)
			{
				match = this.warning.Match(line);
				success = match.Success;
			}

			if (success)
			{
				// handle warning match
				return this.createItemFromMatch(match, CompilerOutputItem.Type.Warning);
			}
			else
			{
				if (this.error != null)
				{
					match = this.error.Match(line);
					success = match.Success;
				}

				if (success)
				{
					// handle error match
					return this.createItemFromMatch(match, CompilerOutputItem.Type.Error);
				}
				else
					return null;
			}
		}

		private CompilerOutputItem createItemFromMatch(Match match, CompilerOutputItem.Type itemType)
		{
			string filename = match.Groups["filename"].Value;
			string _line = match.Groups["line"].Value;
			string _column = match.Groups["column"].Value;
			string message = match.Groups["message"].Value;

			int line = 0;
			int column = 0;

			try
			{
				line = System.Convert.ToInt32(_line);
			}
			catch(FormatException)
			{
				return null;
			}
			catch(OverflowException)
			{
				return null;
			}

			// column is optional
			if (_column != "")
			{
				try
				{
					column = System.Convert.ToInt32(_column);
				}
				catch(FormatException)
				{
					column = 0;
				}
				catch(OverflowException)
				{
					column = 0;
				}
			}

			return new CompilerOutputItem(itemType, filename, line, column, message);
		}

		public string MatchExtra(string line)
		{
			this.CompileRegexps();

			if (this.extraLine != null)
			{
				Match match = extraLine.Match(line);

				if (match.Success)
					return match.Groups["extra"].Value;
				else
					return null;
			}
			else
				return null;
		}
	}
}
