using System;
using System.Diagnostics;
using System.Collections;
using System.Windows.Forms;
using System.Web.Services.Protocols;

namespace DCLJobView.TreeModel
{
	/// <summary>
	///		Models a ExperimentType which 
	///		can have Jobs as child nodes.
	/// </summary>
	public class ExperimentType : DCLJobView.TreeModel.JobViewTreeNode
	{
		private string name;
		private bool exists;
		private int queueCount;

		public ExperimentType(string name, bool exists, JobViewControl jvc)
		{
			this.name = name;
			this.exists = exists;
			this.queueCount = 0;

			if (this.exists)
			{
				this.ImageIndex = JobViewControl.EXPERIMENT_TYPE_IMAGE;
				this.SelectedImageIndex = JobViewControl.EXPERIMENT_TYPE_IMAGE;
			}
			else
			{
				this.ImageIndex = JobViewControl.NON_EXISTING_EXPERIMENT_TYPE_IMAGE;
				this.SelectedImageIndex = JobViewControl.NON_EXISTING_EXPERIMENT_TYPE_IMAGE;
			}

			this.Update(jvc);
		}

		/// <summary>
		///		This name uniquely identifies 
		///		this experiment type.
		/// </summary>
		public string Name
		{
			get { return this.name; }
		}

		/// <summary>
		///		The number of jobs queued 
		///		at this experiment type.
		/// </summary>
		public int QueueCount
		{
			get { return this.queueCount; }
		}

		/// <summary>
		///		It calls HandleTicketList for all of its children (Job).
		/// </summary>
		public override void HandleTicketList(ArrayList ticketList, JobViewControl jvc)
		{
			/* Note: nodes might delete themselves during a HandleTicketList();
					 we must store the list of childnodes in a separate copy to have a safe list */
			TreeNode[] nodesCache = new TreeNode[this.Nodes.Count];
			this.Nodes.CopyTo(nodesCache,0);

			// walk through the copy of the childnode list
			foreach(TreeNode node in nodesCache)
			{
				if(node is Job)
					((Job)node).HandleTicketList(ticketList, jvc);
			}
		}

		public void HandleExperimentList(ArrayList experimentList, JobViewControl jvc)
		{
			bool found = false;

			for(int i=0; i < experimentList.Count; i++)
			{
				// check for our experiment type name
				if (experimentList[i] != null && ((string) experimentList[i]).Equals(this.name))
				{
					found = true;
					// in the rare case a non-existing experiment comes back to life
					if (!this.exists)
					{
						this.exists = true;
						this.ImageIndex = JobViewControl.NON_EXISTING_EXPERIMENT_TYPE_IMAGE;
						this.SelectedImageIndex = JobViewControl.NON_EXISTING_EXPERIMENT_TYPE_IMAGE;
					}

					experimentList.RemoveAt(i);
					this.Update(jvc);

					break;
				}
			}

			// if we know by the info passed to the constructor we do not exist, we don't have to remove us
			// we stay there with a different icon
			if(exists)
			{
				// if we did not find ourself in the list, 
				// we must remove us from the tree
				if(!found) this.Remove();
			}
		}

		/// <summary>
		///		Update all data by calling 
		///		the web service methods.
		/// </summary>
		private void Update(JobViewControl jvc)
		{
			try
			{
				// get job queue count for this experiment type
				this.queueCount = jvc.ExpService.GetQueueofExperiment(this.name);
			}
			catch(SoapException e)
			{
				Trace.WriteLine(e.Message);
				jvc.DoLogMessage("Web Exception: " + e.Message, true);
				this.queueCount = 0;
			}

			// set text of experiment type tree node
			this.Text = this.name + " (queued jobs: " + this.queueCount + ")";
		}

		/// <summary>
		/// Indicates if this experiment type currently exists or if it is only
		/// a placeholder for an older experiment, for which still some jobs exist.
		/// </summary>
		public bool Exists
		{
			get
			{
				return this.exists;
			}
		}
	}
}