using System;
using System.Drawing;
using System.Diagnostics;
using System.Collections;
using System.Windows.Forms;
using System.Web.Services.Protocols;

namespace DCLJobView.TreeModel
{
	/// <summary>
	///		Models a Job in the JobViewControls TreeView.
	/// </summary>
	public class Job : DCLJobView.TreeModel.JobViewTreeNode
	{
		public static Hashtable ResultCodeToColor;

		static Job()
		{
			// TODO: add a setting for this resultcodetocolor mapping
			ResultCodeToColor = new Hashtable();
			ResultCodeToColor.Add("Failed",Color.Red);
			ResultCodeToColor.Add("Experiment crashed",Color.Red);
			ResultCodeToColor.Add("Stop Call Failed",Color.Red);
			ResultCodeToColor.Add("StopSignal",Color.Green);
			ResultCodeToColor.Add("ready",Color.Green);
			ResultCodeToColor.Add("Successful",Color.Green);
			ResultCodeToColor.Add("",Color.Green);
		}

		private string fullstatus;
		private int queueTime;
		private DateTime usageStart;
		private string ticket;

		public Job(string ticket, JobViewControl jvc)
		{
			this.ticket = ticket;
			this.fullstatus = "init";
			this.queueTime = 0;
			this.usageStart = DateTime.MinValue; // NOTE: this looks a bit ugly

			this.ImageIndex = JobViewControl.JOB_RUNNING_IMAGE;
			this.SelectedImageIndex = JobViewControl.JOB_RUNNING_IMAGE;

			this.Update(jvc);
		}

		/// <summary>
		///		Looks for our ticket in the list. If it finds it, it will
		///		update the status information for that ticket/job and remove
		///		the ticket from the list. If it does not find itself in the
		///		list, it will silently remove itself from the tree.
		/// </summary>
		public override void HandleTicketList(ArrayList ticketList, JobViewControl jvc)
		{
			bool found = false;

			for(int i=0; i < ticketList.Count; i++)
			{
				// check for our ticket
				if (ticketList[i] != null && ((string) ticketList[i]).Equals(this.ticket))
				{
					found = true;
					ticketList.RemoveAt(i);
					this.Update(jvc);

					// NOTE: What if the ticket is more than once in the list?
					break;
				}
			}

			// if we did not find ourself in the list, we must remove us from the tree
			if (!found)
			{
				this.Remove();
			}
		}

		/// <summary>
		///		Update all data by calling the web service methods.
		/// </summary>
		private void Update(JobViewControl jvc)
		{
			// retrieve the usageStart only one time
			if (this.usageStart == DateTime.MinValue)
			{
				// get usage start
				string usageStartString = "";
				try
				{
					usageStartString = jvc.TicketService.GetUsageStart(this.ticket);
				}
				catch(SoapException e)
				{
					Trace.WriteLine(e.Message);
					jvc.DoLogMessage("Web Exception: " + e.Message, true);
				}

				// check on error
				if(!usageStartString.Equals(""))
					this.usageStart = DateTime.Parse(usageStartString);
				else
				{
					this.usageStart = DateTime.MinValue;
					jvc.DoLogMessage("No usage start available for job " + this.ticket + ".", true);
				}
			}

			// when the job is already in a finished state, it won't change anymore
			if (!this.Finished)
			{
				// get job status
				try
				{
					this.fullstatus = jvc.ExpService.GetExperimentStatus(this.ticket);
				}
				catch(SoapException e)
				{
					Trace.WriteLine(e.Message);
					jvc.DoLogMessage("Web Exception: " + e.Message, true);
					this.fullstatus = "unknown";
				}

				// if status = queued, get amount of jobs
				// queued in front before your job
				if(this.Status == StatusKind.Queued)
				{
					try
					{
						this.queueTime = jvc.ExpService.GetQueueTime(this.ticket) - 1;
					}
					catch(SoapException e)
					{
						Trace.WriteLine(e.Message);
						jvc.DoLogMessage("Web Exception: " + e.Message, true);
						this.queueTime = 0;
					}
				}

				this.updateIconsAndColors(jvc);
			
				// set node text
				if (this.Status == StatusKind.Queued)
					this.Text = usageStart.ToString() + ": Queued (" + this.queueTime + " jobs in front)";
				else
					this.Text = usageStart.ToString() + ": " + this.BasicStatus + " (" + this.ExtendedResultCode + ")";
			}

			// if the job is at least one day old, we no longer look for results
			TimeSpan timeDiff = DateTime.Now - this.UsageStart;

			// if we do not have any children yet, we have to get the results initially
			if (this.Nodes.Count == 0 || timeDiff.Days < 1)
			{
				// get job results
				string[] results = null;
				try
				{
					results = jvc.ExpService.GetResultList(this.ticket);
				}
				catch(SoapException e)
				{
					Trace.WriteLine(e.Message);
					jvc.DoLogMessage("Web Exception: " + e.Message, true);
				}

				// process job results
				if(results == null)
					jvc.DoLogMessage("No results available for job " + this.ticket, true);
				else
				{
					foreach(string result in results)
					{
						// we must remove \n from the end of the result
						string tmpResult = result.Substring(0, result.Length - 1);

						// result already available?
						bool found = false;
						foreach(Result tmp in this.Nodes)
						{
							if(tmp.Name.Equals(tmpResult)) 
							{
								found = true;
								break;
							}
						}

						// result not available
						if(!found)
						{
							// get result type
							string resultType = null;
							try
							{
								resultType = jvc.ExpService.GetResultType(tmpResult);
							}
							catch(SoapException e)
							{
								Trace.WriteLine(e.Message);
								jvc.DoLogMessage("Web Exception: " + e.Message, true);
							}

							if(resultType != null)
							{
								Result res = new Result(tmpResult, resultType);

								try 
								{
									jvc.Tree.Invoke(new AddNodeHandler(this.addNodeCallback), 
										new object[] { res, this.Nodes });
									if (tmpResult == "Compiler Output")
									{
										if ((jvc.LastSentJobTicket != "" && jvc.LastSentJobTicket == this.ticket)
											|| (jvc.LastSentJobNode != null && jvc.LastSentJobNode == this))
										{
											// send compiler output
											jvc.showCompilerResult(this, res);

											jvc.Tree.SelectedNode = res;
										}
									}
								}
								// this happens in Invoke when no handle is created for the treeview control
								catch(InvalidOperationException)
								{}
							}
						}
					}
				}
			}
		}

		public void updateIconsAndColors(JobViewControl jvc)
		{
			// set icon
			// TODO: icons for compilation and for execution
			if (this.Finished)
			{
				this.ImageIndex = JobViewControl.JOB_FINISHED_IMAGE;
				this.SelectedImageIndex = JobViewControl.JOB_FINISHED_IMAGE;
			}

			// set text color
			if (this.Status == StatusKind.Finished)
				this.ForeColor = jvc.ResultCodeToColor(this.ExtendedResultCode);
			else if (this.Status == StatusKind.Canceled)
				this.ForeColor = Color.DarkGray;
			else if (this.Status == StatusKind.Queued 
				|| this.Status == StatusKind.Compilation
				|| this.Status == StatusKind.Init)
				this.ForeColor = Color.OrangeRed;
			else if (this.Status == StatusKind.Execution)
				this.ForeColor = Color.Blue;
			else
				this.ForeColor = Color.Black;
		}

		private void addNodeCallback(TreeNode node, TreeNodeCollection childs)
		{
			childs.Add(node);
		}
	
		/// <summary>
		///		This ticket uniquely identifies this job.
		/// </summary>
		public string Ticket
		{
			get { return this.ticket; }
		}

		/// <summary>
		///		The full status string as passed from the experiment service.
		///		For detailed information use the properties BasicStatus,
		///		ExtendedResultCode, Status or Finished.
		/// </summary>
		public string FullStatus
		{
			get { return this.fullstatus; }
		}

		/// <summary>
		/// The basic status, such as "init" or "finished". To be sure about the status, better use the properties Status or Finished.
		/// </summary>
		public string BasicStatus
		{
			get
			{
				int k = this.fullstatus.IndexOf("_");

				if (k >= 0)
					return this.fullstatus.Substring(0, k);

				else
					return this.fullstatus;
			}
		}

		/// <summary>
		/// The extended result code provided by the experiment.
		/// </summary>
		public string ExtendedResultCode
		{
			get
			{
				int k = this.fullstatus.IndexOf("_");

				if (k >= 0)
					return this.fullstatus.Substring(k+1);

				else
					return "";
			}
		}

		public enum StatusKind {Init, Queued, Compilation, Execution, Finished, Canceled, Unknown};

		/// <summary>
		/// One of some job stati.
		/// </summary>
		public StatusKind Status
		{
			get
			{
				string s = this.BasicStatus;

				if (s == "init")
					return StatusKind.Init;
				else if (s == "queued")
					return StatusKind.Queued;
				else if (s == "compilation")
					return StatusKind.Compilation;
				else if (s == "execution")
					return StatusKind.Execution;
				else if (s == "finished" || s == "Finish")
					return StatusKind.Finished;
				else if (s == "canceled")
					return StatusKind.Canceled;
				else if (s == "unknown")
					return StatusKind.Unknown;
				else
					return StatusKind.Unknown;
			}
		}

		/// <summary>
		///		True if the job is in a finished state (finished or canceled).
		/// </summary>
		public bool Finished
		{
			get
			{
				return ( this.Status == StatusKind.Finished
						|| this.Status == StatusKind.Canceled );
			}
		}

		/// <summary>
		///		The number of jobs queued before this one.
		/// </summary>
		public int QueueTime
		{
			get { return this.queueTime; }
		}	

		/// <summary>
		///		The timestamp of the time the ticket was marked as used.
		/// </summary>
		public DateTime UsageStart
		{
			get { return this.usageStart; }
		}
	}
}